using System;
using System.Collections.Generic;
using System.Text;
using DartScoreboard.GUI.Controls;

namespace DartScoreboard.Data
{
    public class BaseGame
    {

        public delegate void RoundChangedDelegate(int round);
        public event RoundChangedDelegate RoundChanged;

        //public delegate void LastRoundDelegate(int round);
        //public event LastRoundDelegate LastRound;

        //public delegate void RoundLimitDelegate(PlayerController player);
        //public event RoundLimitDelegate RoundLimit;

        public delegate void GameWonDelegate(PlayerController player);
        public event GameWonDelegate GameWon;

        private const int MAX_ROUNDS = 30;

        private List<PlayerController> _players;
        private int _currentPlayerIndex;
        private int _round;
        private DartsClosed _dartsClosed;
        private int _highScore = 0;
        private Stack<DartHistoryItem> _dartHistory = null;
        private DateTime _startTime;


        public List<PlayerController> Players
        {
            get { return _players; }
            set { _players = value; }
        }

        public int Round
        {
            get { return _round; }
            set 
            { 
                _round = value;

               OnRoundChanged();
            }
        }



        public int CurrentPlayerIndex
        {
            get { return _currentPlayerIndex; }
            set { _currentPlayerIndex = value; }
        }

        public BaseGame()
        {
            _players = new List<PlayerController>();
            _currentPlayerIndex = 0;
            _round = 1;
            _dartsClosed = new DartsClosed();
            _dartHistory = new Stack<DartHistoryItem>();
            _startTime = DateTime.Now;
        }

        public void OnPlayerChange()
        {
            if (_currentPlayerIndex + 1 < _players.Count)
            {
                _currentPlayerIndex++;
            }
            else
            {
                _currentPlayerIndex = 0;
                Round++;
            }

            _players[_currentPlayerIndex].StartTurn();
        }

        public int UndoLastDart()
        {
            if (_dartHistory.Count > 0)
            {
                DartHistoryItem dartHistoryItem = _dartHistory.Pop();


                _dartsClosed[dartHistoryItem.Dart] = dartHistoryItem.IsClosed;
                _players[dartHistoryItem.PlayerIndex].Score -= dartHistoryItem.Score;
                _players[dartHistoryItem.PlayerIndex].UnMarkDart(dartHistoryItem.Dart, dartHistoryItem.DartMarks, dartHistoryItem.NumberThrown);


                if (_currentPlayerIndex > dartHistoryItem.PlayerIndex || dartHistoryItem.PlayerIndex + 1 == _players.Count)
                {
                    _players[_currentPlayerIndex].UndoTurn();
                }

                _currentPlayerIndex = dartHistoryItem.PlayerIndex;
                _round = dartHistoryItem.Round;



                //Update High Scores
                UpdateHighScore();

                //return the round in case it has changed
            }

            return _round;
        }

        private void UpdateHighScore()
        {
            int highScore = 0;

            foreach (PlayerController player in _players)
            {
                if (player.Score > highScore)
                {
                    highScore = player.Score;
                }
            }
        }

        private int UpdateScores(Darts lastDartThrown, int numberThrown)
        {
            int scoreToAdd = 0;
 
            if (!_dartsClosed[lastDartThrown])
            {
                //can still score
                int numberOpen = 3 - _players[_currentPlayerIndex].DartTracker[lastDartThrown];
                int numberScoring = 0;

                if (numberOpen > 0)
                {
                    numberScoring = numberThrown - numberOpen;
                }
                else
                {
                    numberScoring = numberThrown;
                }

                if (numberScoring > 0 && !CheckClosedStatus(lastDartThrown))
                {
                    scoreToAdd = (int)lastDartThrown * numberScoring;
                    _players[_currentPlayerIndex].Score += scoreToAdd;
                }
            }

            if (_players[_currentPlayerIndex].Score > _highScore)
            {
                _highScore = _players[_currentPlayerIndex].Score;
            }

            return scoreToAdd;
        }


        private bool CheckClosedStatus(Darts lastDartThrown)
        {
            bool closed = true;

            foreach (PlayerController player in _players)
            {
                if (player != _players[CurrentPlayerIndex])
                {
                    if (player.DartTracker[lastDartThrown] < 3)
                    {
                        closed = false;
                        break;
                    }
                }
            }

            return closed;
        }

        private void UpdateClosedStatus(Darts lastDartThrown)
        {
            bool closed = true;

            foreach (PlayerController player in _players)
            {
                if (player.DartTracker[lastDartThrown] < 3)
                {
                    closed = false;
                    break;
                }
            }

            if (closed)
            {
                _dartsClosed[lastDartThrown] = true;
            }

        }

        private void OnGameFinished()
        {

            SaveGame();

            if (GameWon != null)
                GameWon(_players[_currentPlayerIndex]);
        }

        private void SaveGame()
        {
            DartDataSetTableAdapters.DartGameTableAdapter dgTA = new DartDataSetTableAdapters.DartGameTableAdapter();
            DartDataSetTableAdapters.DartGamePlayersTableAdapter dgpTA = new DartScoreboard.DartDataSetTableAdapters.DartGamePlayersTableAdapter();
            int gameID = dgTA.Insert(_players[_currentPlayerIndex].ID, _startTime, DateTime.Now);

            foreach (PlayerController player in _players)
            {
                dgpTA.Insert(gameID, player.ID, player.DartsThrownTotal, player.GetCountDartsScoring(), player.Score);
            }
        }

        private void OnRoundChanged()
        {
            if (RoundChanged != null)
                RoundChanged(_round);
        }

        private bool IsGameOver()
        {
            if (_players[_currentPlayerIndex].DartTracker[Darts.Dart20] >= 3 &&
                _players[_currentPlayerIndex].DartTracker[Darts.Dart19] >= 3 &&
                _players[_currentPlayerIndex].DartTracker[Darts.Dart18] >= 3 &&
                _players[_currentPlayerIndex].DartTracker[Darts.Dart17] >= 3 &&
                _players[_currentPlayerIndex].DartTracker[Darts.Dart16] >= 3 &&
                _players[_currentPlayerIndex].DartTracker[Darts.Dart15] >= 3 &&
                _players[_currentPlayerIndex].DartTracker[Darts.DartBull] >= 3 &&
                _players[_currentPlayerIndex].Score >= _highScore)
            {
                return true;
            }

            return false;
        }

        public void UpdateGame(Darts lastDartThrown, int numberThrown)
        {
            int score = 0;
            bool wasClosed = _dartsClosed[lastDartThrown];

            if (lastDartThrown != Darts.DartOutOfBounds)
            {
                score = UpdateScores(lastDartThrown, numberThrown);
                UpdateClosedStatus(lastDartThrown);
            }

            List<int> dartMarks = _players[_currentPlayerIndex].MarkDart(lastDartThrown, numberThrown);

            _dartHistory.Push(new DartHistoryItem(score, lastDartThrown, numberThrown, wasClosed, _currentPlayerIndex, _round, dartMarks));

            bool isGameOver = IsGameOver();

            if (!isGameOver)
            {
                if (_players[_currentPlayerIndex].DartsRemaining == 0)
                {
                    OnPlayerChange();
                }
            }
            else
            {
                OnGameFinished();
            }
        }
    }
}
