Imports System
'using System.Math;
Imports System.Globalization
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes

Namespace DartScoreboardVB.Animations

    '/ <summary>
    '/ BounceDoubleAnimation
    '/ </summary>
    Public Class BounceDoubleAnimation
        Inherits DoubleAnimationBase
        Public Enum EdgeBehaviorEnum
            EaseIn
            EaseOut
            EaseInOut
        End Enum

#Region "Dependency Properties"
        Public Shared EdgeBehaviorProperty As DependencyProperty = _
            DependencyProperty.Register("EdgeBehavior", _
                GetType(EdgeBehaviorEnum), _
                GetType(BounceDoubleAnimation), _
                New PropertyMetadata(EdgeBehaviorEnum.EaseInOut))

        Public Shared BouncesProperty As DependencyProperty = _
            DependencyProperty.Register("Bounces", _
                GetType(Integer), _
                GetType(BounceDoubleAnimation), _
                New PropertyMetadata(5))

        Public Shared BouncinessProperty As DependencyProperty = _
            DependencyProperty.Register("Bounciness", _
                GetType(Double), _
                GetType(BounceDoubleAnimation), _
                New PropertyMetadata(2.0))

        Public Shared SrcProperty As DependencyProperty = _
            DependencyProperty.Register("Src", _
                GetType(Double), _
                GetType(BounceDoubleAnimation), _
                New PropertyMetadata(Nothing))

        Public Shared DestProperty As DependencyProperty = _
            DependencyProperty.Register("Dst", _
                GetType(Double), _
                GetType(BounceDoubleAnimation), _
                New PropertyMetadata(Nothing))
#End Region
        '/ <summary>
        '/ Specifies which side of the transition gets the "bounce" effect.
        '/ </summary>
        Public Property EdgeBehavior() As EdgeBehaviorEnum
            Get
                Return CType(GetValue(EdgeBehaviorProperty), EdgeBehaviorEnum)
            End Get
            Set(ByVal Value As EdgeBehaviorEnum)
                SetValue(EdgeBehaviorProperty, Value)
            End Set
        End Property
        '/ <summary>
        '/  Number of bounces in the effect
        '/ </summary>
        Public Property Bounces() As Integer
            Get
                Return CType(GetValue(BouncesProperty), Integer)
            End Get
            Set(ByVal Value As Integer)
                If Value > 0 Then
                    SetValue(BouncesProperty, Value)
                Else
                    Throw New ArgumentException("can't set the bounces to " + Value)
                End If
            End Set
        End Property

        '/ <summary>
        '/ Specifies the amount by which the element springs back.
        '/ </summary>
        Public Property Bounciness() As Double
            Get
                Return CType(GetValue(BouncinessProperty), Double)
            End Get
            Set(ByVal Value As Double)
                If Value > 0 Then
                    SetValue(BouncinessProperty, Value)
                Else
                    Throw New ArgumentException("can't set the bounciness to " + Value)
                End If
            End Set
        End Property

        '/ <summary>
        '/ Specifies the starting value of the animation.
        '/ </summary>
        Public Property Src() As Double
            Get
                Return CType(GetValue(SrcProperty), Double)
            End Get
            Set(ByVal Value As Double)
                SetValue(SrcProperty, Value)
            End Set
        End Property

        '/ <summary>
        '/ Specifies the ending value of the animation.
        '/ </summary>
        Public Property Dest() As Double
            Get
                Return CType(GetValue(DestProperty), Double)
            End Get
            Set(ByVal Value As Double)
                SetValue(DestProperty, Value)
            End Set
        End Property

        Protected Overrides Function GetCurrentValueCore( _
        ByVal defaultOriginValue As Double, _
        ByVal defaultDestinationValue As Double, _
        ByVal clock As AnimationClock) As Double
            Dim ReturnValue As Double
            Dim start As Double = IIf(Not Src.Equals(Nothing), CType(Src, Double), defaultOriginValue)
            Dim delta As Double = IIf(Not Dest.Equals(Nothing), CType(Dest, Double) - start, defaultOriginValue - start)

            Select Case Me.EdgeBehavior
                Case EdgeBehaviorEnum.EaseIn
                    ReturnValue = easeIn(clock.CurrentProgress.Value, start, delta, Bounciness, Bounces)
                    Exit Function
                Case EdgeBehaviorEnum.EaseOut
                    ReturnValue = easeOut(clock.CurrentProgress.Value, start, delta, Bounciness, Bounces)
                    Exit Function
                Case EdgeBehaviorEnum.EaseInOut
                Case Else
                    ReturnValue = easeInOut(clock.CurrentProgress.Value, start, delta, Bounciness, Bounces)
                    Exit Function
            End Select
            Return ReturnValue
        End Function

        Protected Overrides Function CreateInstanceCore() As Freezable
            Return New BounceDoubleAnimation()
        End Function


        Private Shared Function easeOut(ByVal timeFraction As Double, ByVal start As Double, ByVal delta As Double, ByVal bounciness As Double, ByVal bounces As Integer) As Double
            Dim ReturnValue As Double = 0.0

            ' math magic: The cosine gives us the right wave, the timeFraction is the frequency of the wave, 
            ' the absolute value keeps every value positive (so it "bounces" off the midpoint of the cosine 
            ' wave, and the amplitude (the exponent) makes the sine wave get smaller and smaller at the end.
            ReturnValue = Math.Abs(Math.Pow((1 - timeFraction), bounciness) _
                          * Math.Cos(2 * Math.PI * timeFraction * bounces))
            ReturnValue = delta - (ReturnValue * delta)
            ReturnValue += start
            Return ReturnValue
        End Function

        Private Shared Function easeIn(ByVal timeFraction As Double, ByVal start As Double, ByVal delta As Double, ByVal bounciness As Double, ByVal bounces As Integer) As Double
            Dim ReturnValue As Double = 0.0
            ' math magic: The cosine gives us the right wave, the timeFraction is the amplitude of the wave, 
            ' the absolute value keeps every value positive (so it "bounces" off the midpoint of the cosine 
            ' wave, and the amplitude (the exponent) makes the sine wave get bigger and bigger towards the end.
            ReturnValue = Math.Abs(Math.Pow((timeFraction), bounciness) _
                          * Math.Cos(2 * Math.PI * timeFraction * bounces))
            ReturnValue = ReturnValue * delta
            ReturnValue += start
            Return ReturnValue
        End Function

        Private Shared Function easeInOut(ByVal timeFraction As Double, ByVal start As Double, ByVal delta As Double, ByVal bounciness As Double, ByVal bounces As Integer) As Double
            Dim ReturnValue As Double = 0.0

            ' we cut each effect in half by multiplying the time fraction by two and halving the distance.
            If timeFraction <= 0.5 Then
                ReturnValue = easeIn(timeFraction * 2, start, delta / 2, bounciness, bounces)
            Else
                ReturnValue = easeOut((timeFraction - 0.5) * 2, start, delta / 2, bounciness, bounces)
                ReturnValue += delta / 2
            End If
            Return ReturnValue
        End Function

    End Class
End Namespace

